/**
 *
 */
package jp.veritrans.tercerog.sample.util;

import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.UnknownHostException;

import jp.veritrans.tercerog.mdk.dto.MdkServiceConstants;

/**
 * キャリア用ユーティリティクラス
 * 
 * @author t-hatakeyama
 * 
 */
public class CarrierUtility {

    /**
     * インスタンス
     */
    public static CarrierUtility instance = new CarrierUtility();

    /** docomoのIPアドレス一覧(一部) */
    private static String[] docomoIpAddressList = { "1.66.96.0/21", "1.66.104.0/23", "1.72.0.0/14", "1.76.0.0/18", };
    /** auのIPアドレス一覧(定義のみ) */
    private static String[] auIpAddressList = {};

    /**
     * コンストラクタ
     */
    private CarrierUtility()
    {
        // propertiesファイルからIPアドレスを取得する
    }

    /**
     * IPアドレスからキャリアを判別します
     * 
     * @param ipAddress XX.XX.XX.XX 形式の文字列
     * @return "docomo"/"au"/"softbank"
     */
    public String findCarrierFromIpAddress(final String ipAddress)
    {
        // ドコモのIPアドレス比較
        if (existCarrierAddressList(ipAddress, docomoIpAddressList)) {
            return MdkServiceConstants.CARRIER_DOCOMO;
        }

        // auのIPアドレス比較
        if (existCarrierAddressList(ipAddress, auIpAddressList)) {
            return MdkServiceConstants.CARRIER_AU;
        }

        // softbankは支払方法が複数あるため、IPアドレスのチェックで判別できない。

        return null;
    }

    /**
     * HTMLヘッダから端末種別を取得する
     * 
     * @param userAgent HTMLヘッダのuser-agent部分
     * @return "1":スマートフォン、"2":フィーチャーフォン（ガラケー）
     */
    public String getTerminalKind(final String userAgent)
    {
        // 文字列比較により、端末種別を決める
        if (userAgent.indexOf("DoCoMo") == 0 // docomo
                || userAgent.indexOf("J-PHONE") == 0 // softbank
                || userAgent.indexOf("Vodafone") == 0
                || userAgent.indexOf("SoftBank") == 0
                || userAgent.indexOf("UP.Browser") == 0 // au
                || userAgent.indexOf("KDDI") == 0) {
            // user-agentの開始文字が上記の場合に、フィーチャフォンと判断する
            return "2";
        } else if (userAgent.indexOf("iPhone") > 0 && userAgent.indexOf("iPad") == -1
                || userAgent.indexOf("iPod") > 0
                || userAgent.indexOf("Android") > 0) {
            // user-agentの文字列の中に上記がある場合に、 スマートフォンと判断する
            return "1";
        }

        // それ以外（PC）
        return null;
    }

    /**
     * アドレスリストの中に該当するアドレスがあるかチェックする
     * 
     * @param ipAddress
     * @param carrierIpAddressList
     * @return
     */
    private boolean existCarrierAddressList(final String ipAddress, final String[] carrierIpAddressList)
    {
        // キャリアのIPアドレス一覧と指定されたIPアドレスのネットワークアドレス部が一致するかチェックします
        for (int i = 0; i < carrierIpAddressList.length; i++) {
            String carrierIpAddress = carrierIpAddressList[i];
            if (ipEquals(ipAddress, carrierIpAddress)) {
                return true;
            }
        }

        return false;
    }

    /**
     * IPアドレスのネットワークアドレス部分の比較
     * 
     * @param localIp XX.XX.XX.XX 形式の文字列
     * @param carrierIp XX.XX.XX.XX/YY 形式の文字列
     * @return
     */
    private boolean ipEquals(final String localIp, final String carrierIp)
    {
        // XX.XX.XX.XX/YY のIPアドレス部とネットワークアドレス長を分けます
        String[] splitCarrierIp = carrierIp.split("/");

        // ネットワークアドレス部を比較するためにシフトする長さを求めます。
        int maskBitLength = Integer.parseInt(splitCarrierIp[1]);
        int hostBitLength = 32 - maskBitLength;

        // IPアドレスのネットワークアドレス部のみを取得します。
        int intLocalIp = ip2Int(localIp);
        if (intLocalIp == -1) {
            return false;
        }
        intLocalIp = intLocalIp >> hostBitLength << hostBitLength;
        int intCarrierIp = ip2Int(splitCarrierIp[0]) >> hostBitLength << hostBitLength;

        // ネットワークアドレス部の比較をします。
        return intLocalIp == intCarrierIp;
    }

    /**
     * IPアドレス（XX.XX.XX.XX）をint型に変更する
     * 
     * @param address XX.XX.XX.XX 形式の文字列
     * @return 変換したIPアドレス
     */
    private int ip2Int(final String ip)
    {
        InetAddress address = null;
        try {
            address = InetAddress.getByName(ip);
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }

        if (address instanceof Inet4Address) {
            // IPv4の場合
            String[] splitIp = ip.split("\\.");
            int b0 = Integer.parseInt(splitIp[0]) << 24;
            int b1 = Integer.parseInt(splitIp[1]) << 16;
            int b2 = Integer.parseInt(splitIp[2]) << 8;
            int b3 = Integer.parseInt(splitIp[3]);

            return b0 | b1 | b2 | b3;
        } else if (address instanceof Inet6Address) {
            // IPv6の場合、比較しない
        }

        return -1;
    }

    /**
     * エンコーディングする文字列を返す
     * ・terminalKindがフィーチャーフォンの場合はShift_JIS固定
     * ・キャリアが、docomo,sbpsの場合はShift_JIS固定
     * ・上記以外はUTF-8
     * 
     * @param carrier
     * @param terminalKind
     * @return
     */
    public static String getEncoding(final String carrier, final String terminalKind) {
        if (terminalKind.equals(MdkServiceConstants.CARRIER_TERMINAL_FP)) {
            return "Shift_JIS";
        }

        if (carrier.equals(MdkServiceConstants.CARRIER_DOCOMO)
                || carrier.equals(MdkServiceConstants.CARRIER_SB_KTAI)) {
            return "Shift_JIS";
        }
        return "UTF-8";
    }
}
