package jp.veritrans.tercerog.sample.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLDecoder;
import java.text.MessageFormat;
import java.util.Properties;

public class Utils {

    /**
     * HTML出力できる状態にする。
     * パラメータがnullの場合は空文字に変更する。
     *
     * @param str nullの可能性がある文字列
     * @return nullを空文字に変更した文字列
     */
    public static String out(final Object str) {
        if (str == null) {
            return "";
        }
        return str.toString();
    }

    /**
     * HTMLの属性値として出力できる状態にする。
     *
     * <ul>
     * <li>パラメータがnullの場合は空文字に変更する。</li>
     * <li>&quot;を&amp;quot;に変換する。</li>
     * </ul>
     *
     * @param str 出力する文字列
     * @return  HTMLの属性値として整形された文字列
     */
    public static String outForAttribute(final Object str) {
        String outStr = out(str);
        return outStr.replaceAll("\"", "&quot;");
    }

    /**
     * パラメータがnull(空文字を含む)でないかをチェックする。
     *
     * @param param 検査したい文字列。
     * @return nullの場合はtrueを返す。
     */
    public static boolean isNull(final Object param) {
        if (param == null || param.toString().length() == 0) {
            return true;
        }
        return false;
    }

    /**
     * 接続先名／接続先IPアドレスの形式の文字列を返す。
     *
     * @param contextPath
     * @return
     */
    public static String getHostName(final String configPath) {
        Properties config = new Properties();
        try {
            config.load(new FileInputStream(new File(configPath)));

            String serverUrl = config.getProperty("HOST_URL");
            URI uri = new URI(serverUrl);
            String host = uri.getHost();

            return host + "／" + InetAddress.getByName(host).getHostAddress();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 接続先名／接続先IPアドレスの形式の文字列を返す。
     *
     * @param contextPath
     * @return
     */
    public static String getDummyMode(final String configPath) {
        Properties config = new Properties();
        try {
            config.load(new FileInputStream(new File(configPath)));

            String dummyRequest = config.getProperty("DUMMY_REQUEST");

            if ("1".equals(dummyRequest)) {
                return "ダミーリクエスト";
            } else {
                return "本番リクエスト";
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 接続先の文字列を返す。
     *
     * @param contextPath
     * @return
     */
    public static String getHost(final String configPath) {
        Properties config = new Properties();
        try {
            config.load(new FileInputStream(new File(configPath)));

            String serverUrl = config.getProperty("HOST_URL");
            URI uri = new URI(serverUrl);

            return uri.getScheme() + "://" + uri.getHost();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * URLエンコードされた文字列をデコードする。
     *
     * @param str エンコードされた文字列
     * @return デコード後の文字列
     */
    public static String urlDecode(final String str) {
        try {
            return URLDecoder.decode(new String(str.getBytes("ISO-8859-1"), "UTF-8"), "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 日時（文字列）を表示用に編集する。
     *
     * 例）
     * 日時が「20130425」（8桁）の場合、「2013-04-25」
     * 日時が「20130425103758」（14桁）の場合、「2013-04-25 10:37:58」
     * 日時が「20130425103758741」（17桁）の場合、「2013-04-25
     * 10:37:58.741」
     *
     * @param str
     * @return
     */
    public static String formatDateTime(final String str) {
        MessageFormat dateFormat08 = new MessageFormat("{0}-{1}-{2}"); // yyyy-mm-ddフォーマット
        MessageFormat dateFormat14 = new MessageFormat("{0}-{1}-{2} {3}:{4}:{5}"); // yyyy-mm-dd hh:mi:ssフォーマット
        MessageFormat dateFormat17 = new MessageFormat("{0}-{1}-{2} {3}:{4}:{5}.{6}"); // yyyy-mm-dd hh:mi:ss.sssフォーマット
        String result = "";

        String sy = ""; // 年
        String sm = ""; // 月
        String sd = ""; // 日
        String sh = ""; // 時
        String si = ""; // 分
        String ss = ""; // 秒
        String sss = ""; // 秒以下

        switch (str.length()) {
            case 8:
                sy = str.substring(0, 4);
                sm = str.substring(4, 6);
                sd = str.substring(6, 8);
                result = dateFormat08.format(new Object[] { sy, sm, sd });
                break;
            case 14:
                sy = str.substring(0, 4);
                sm = str.substring(4, 6);
                sd = str.substring(6, 8);
                sh = str.substring(8, 10);
                si = str.substring(10, 12);
                ss = str.substring(12, 14);
                result = dateFormat14.format(new Object[] { sy, sm, sd, sh, si, ss });
                break;
            case 17:
                sy = str.substring(0, 4);
                sm = str.substring(4, 6);
                sd = str.substring(6, 8);
                sh = str.substring(8, 10);
                si = str.substring(10, 12);
                ss = str.substring(12, 14);
                sss = str.substring(14, 17);
                result = dateFormat17.format(new Object[] { sy, sm, sd, sh, si, ss, sss });
                break;
            default:
                result = str;
                break;
        }

        return result;
    }
}
